@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# $Id: CloudServices.py 1233 2021-09-07 03:22:45Z Casey $
+# $Id: CloudServices.py 1243 2021-09-09 10:41:54Z Casey $
 #
 # Copyright (c) 2021 Nuwa Information Co., Ltd, All Rights Reserved.
 #
@@ -12,8 +12,8 @@
 # limitations under the License.
 #
 # $Author: Casey $
-# $Date: 2021-09-07 11:22:45 +0800 (Tue, 07 Sep 2021) $
-# $Revision: 1233 $
+# $Date: 2021-09-09 18:41:54 +0800 (Thu, 09 Sep 2021) $
+# $Revision: 1243 $
 
 import json
 import socket
@@ -49,7 +49,11 @@ class BaseServices(object):
     
     name = "Base"
 
-    def register(self, managerEmail, ifttt, companyName, projectName, server, domain, aliasDomainList=None, deleteDnsAndHealthchecks=False):
+    def register(self, managerEmail, ifttt, companyName, projectName, server, domain, aliasDomainList=None):
+        return {
+            
+        }
+    def unregister(self, managerEmail, ifttt, companyName, projectName, server, domain, aliasDomainList=None):
         return {
             
         }
@@ -59,7 +63,7 @@ class CloudflareService(BaseServices):
 
     name = "Cloudflare DNS"
 
-    def register(self, managerEmail, ifttt, companyName, projectName, server, domain, aliasDomainList=None, deleteDnsAndHealthchecks=False):
+    def register(self, managerEmail, ifttt, companyName, projectName, server, domain, aliasDomainList=None):
         # cf
         cf = Cloudflare(CLOUDFLARE_EMAIL, CLOUDFLARE_API_KEY)
         subdomain = tldextract.extract(domain).subdomain
@@ -104,35 +108,77 @@ class CloudflareService(BaseServices):
                     result = result[0]
                 else:
                     result = result[0]
-                    if deleteDnsAndHealthchecks: # delete
-                        result = cf.delete_record(zoneID, result["id"], domain, result['ttl'], ip, result['proxied'])
-                        result = result["result"]
-                        finalDeleteResult.append(domain)
-                    else: # update
-                        result = cf.update_record(zoneID, result["id"], domain, result['ttl'], ip, result['proxied'])
-                        result = result["result"]
+                    # update
+                    result = cf.update_record(zoneID, result["id"], domain, result['ttl'], ip, result['proxied'])
+                    result = result["result"]
 
                 finalResult.append(result)
 
-            if finalDeleteResult:
-                deletedDomains = ''
-                for domain in finalDeleteResult:
-                    deletedDomains += f'{domain}, '
-                return {'dns records have been deleted': deletedDomains}
-            else:
-                name = ''
-                for result in finalResult:
-                    name += f'{result.get("name")}, '
+            name = ''
+            for result in finalResult:
+                name += f'{result.get("name")}, '
 
-                return {
-                    'zone name': finalResult[0].get("zone_name"), 
-                    'name': name, 
-                    'type': finalResult[0].get("type"), 
-                    'content': finalResult[0].get("content")
-                }
+            return {
+                'zone name': finalResult[0].get("zone_name"), 
+                'name': name, 
+                'type': finalResult[0].get("type"), 
+                'content': finalResult[0].get("content")
+            }
             
         raise RuntimeError("Can't get ip of server and zoneID: %s" % server)
 
+    def unregister(self, managerEmail, ifttt, companyName, projectName, server, domain, aliasDomainList=None):
+        # cf
+        cf = Cloudflare(CLOUDFLARE_EMAIL, CLOUDFLARE_API_KEY)
+        subdomain = tldextract.extract(domain).subdomain
+        zone = '%s.%s' % (tldextract.extract(domain).domain, tldextract.extract(domain).suffix)
+        
+        # zone id
+        response = cf.zones(zone)
+        
+        result = response.get("result")
+        if result:
+            zoneID = result[0].get("id", None)
+        else:
+            zoneID = None
+            
+        if not zoneID:
+            raise RuntimeError("Zone not in Cloudflare: %s" % zone)
+        
+        # get server ip
+        try:
+            ip = socket.gethostbyname(server)
+        except Exception as e:
+            raise RuntimeError("Can't get ip of server: %s" % server)
+
+        domains = []
+        domains.append(domain)
+        if aliasDomainList:
+            for alias in aliasDomainList:
+                domains.append(alias)
+
+        # check dns 
+        if ip and zoneID:
+            finalDeleteResult = []
+
+        for domain in domains:
+            result = cf.dns_records(zoneID, domain).get('result')
+            if not result:
+                return {'Dns records have not been added before': domain}
+            else:
+                result = result[0]
+                result = cf.delete_record(zoneID, result["id"], domain, result['ttl'], ip, result['proxied'])
+                result = result["result"]
+                finalDeleteResult.append(domain)
+
+        if finalDeleteResult:
+            deletedDomains = ''
+            for domain in finalDeleteResult:
+                deletedDomains += f'{domain}, '
+            return {'Dns records have been deleted': deletedDomains}
+        else:
+            return {'There is no dns records need to be delete': ''}
+
 class HealthchecksService(BaseServices):
 
     name = "Healthchechs DNS"
@@ -399,7 +445,7 @@ class HealthchecksService(BaseServices):
             browser.select_form(f'form[action="/checks/{checkUrlId}/remove/"]')
             browser.submit_selected()
 
-    def register(self, managerEmail, ifttt, companyName, projectName, server, domain, aliasDomainList=None, deleteDnsAndHealthchecks=False):
+    def register(self, managerEmail, ifttt, companyName, projectName, server, domain, aliasDomainList=None):
         # connect 
         browser = self.loginHealthchecks()
        
@@ -416,12 +462,6 @@ class HealthchecksService(BaseServices):
             
         projectLink = projectLink.replace("checks/", "") # /EX: projects/9e7208d8-8b72-4872-8859-b781ae952b60/checks/
 
-        if deleteDnsAndHealthchecks:
-            self.deleteHealthchecks(browser, projectLink)
-            return {
-                'Deleted Healthchecks': hchkProjectName, 
-            }
-
         # create API
         browser.open("%s/%ssettings" % (HEALTHCHECKS_URL, projectLink))
         for div in browser.page.select("div.panel-body"):
@@ -454,7 +494,30 @@ class HealthchecksService(BaseServices):
             'Document': "Please see Deploy document '如何執行排程程式(Crontab 寫法)' and set API key to settings.py.",
             "Suggestion": "Please go to %s to check emails." % managerEmail,
         }
+
+    def unregister(self, managerEmail, ifttt, companyName, projectName, server, domain, aliasDomainList=None):
+        # connect 
+        browser = self.loginHealthchecks()
+       
+        # find project
+        hchkProjectName = "[%s] %s" % (companyName, projectName)
+        projectLink = self.findProject(browser, hchkProjectName)
+
+        if not projectLink:
+            browser.select_form('#add-project-modal form')
+            browser["name"] = hchkProjectName
+            browser.submit_selected()
+            
+            projectLink = self.findProject(browser, hchkProjectName)
+            
+        projectLink = projectLink.replace("checks/", "") # /EX: projects/9e7208d8-8b72-4872-8859-b781ae952b60/checks/
+
+        self.deleteHealthchecks(browser, projectLink)
         
+        return {
+            'Deleted Healthchecks': hchkProjectName, 
+        }
+
 class SentryService(BaseServices):
 
     name = "Sentry DSN"
@@ -499,7 +562,7 @@ class SentryService(BaseServices):
         browser["email"] = managerEmail
         browser.submit_selected()
 
-    def register(self, managerEmail, ifttt, companyName, projectName, server, domain, aliasDomainList=None, deleteDnsAndHealthchecks=False):
+    def register(self, managerEmail, ifttt, companyName, projectName, server, domain, aliasDomainList=None):
         # connect 
         browser = self.loginSentry()
         
